#!/usr/bin/env python
# -*- coding: utf-8 -*-

#from notmm.utils.urlmap import RegexURLMap, url
from django.conf.urls import url
from notmm.utils.template import direct_to_template
from sandbox_sqlalchemy import views

# a custom/generic login view
handleLogin = 'blogengine2.contrib.session.login'
handleLogout = 'blogengine2.contrib.session.logout'

urlpatterns = [
        url(r'^test_with_session/$', views.test_with_session)
]
# Add the authentication views
#urlpatterns.add_routes('',
        # Custom authentication backend wrapper (authkit based)
        # see nomainapp..controllers.auth.LoginController
        #url(r'^session_login/$', handleLogin, \
        #    dict(template_name='auth/login.mako')),
        #url(r'^session_logout/$', handleLogout, \
        #    dict(template_name='auth/logout.mako')),
#)

#urlpatterns.include('sandbox_sqlalchemy.config.extras')

#blogengine api v1
#urlpatterns.include('blogengine.contrib.api_v1.urls', prefix="^blog/")


