{% load i18n %}
{% load satchmo_currency satchmo_adminorder_tags %}
    <div class="module" id="Orders">
		<h2 class="module-title">{% trans "Satchmo Orders" %}</h2>
		<div class="module-content">
        	<h2><a href="shop/order/?status__exact=New">{% trans "New Orders" %}</a></h2>
			{% pending_order_list %}
	        <h2><a href="shop/order/?status__exact=In%20Process">{% trans "Orders in Process" %}</a></h2>
			{% inprocess_order_list %}
	        <h2><a href="shop/order/">{% trans "View all Orders" %}</a></h2>
	    </div>
	</div>
