from notmm.utils.urlmap import RegexURLMap

urlpatterns = RegexURLMap()

#urlpatterns.set_default_prefix('/foo/')

urlpatterns.add_routes('sandbox.views.decorators', 
        ### decorators tests
        ### with_schevo_database test
        (r'^test_with_schevo_database$', 'test_schevo_database', {
            'template_name' : 'sandbox/test.mako'
        }),
)

urlpatterns.add_routes('sandbox.handlers',
        (r'^moved/$', 'handle302', {
            'location' : '/index.html'
        }),
        (r'^croak$', 'handle500')
)
#urlpatterns.commit()

