# Copyright (C) 2004,2005 by SICEm S.L. and Imendio AB
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from gazpacho.properties import prop_registry, CustomProperty, BooleanType

class HasResizeGripProp(CustomProperty, BooleanType):
    label = "Has Resize Grip"
    default = True
    def get(self):
        return self.object.get_has_resize_grip()

    def set(self, value):
        self.object.set_has_resize_grip(value)

prop_registry.override_simple('GtkStatusbar::has-resize-grip',
                              HasResizeGripProp)

