{% extends "admin/base_site.html" %}
{% load i18n satchmo_util adminmedia %}

{% block breadcrumbs %}{% if not is_popup %}
<div class="breadcrumbs">
     <a href="/admin/">{% trans "Home" %}</a> &rsaquo;
     <a href="{% url satchmo_admin_variation_list %}">{% trans 'Product Variation Manager' %}</a>
</div>
{% endif %}{% endblock %}

{% block content %}
<h1>{% trans 'Select the Configurable Product to manage.' %}</h1>
<ul>
{% for p in products %}
    <li>
        <a href="{% url satchmo_admin_variation_manager p.id %}">{{ p.translated_name }}</a>
    </li>
{% endfor %}
</ul>
{% endblock %}
