from notmm.utils.urlmap import RegexURLMap, url

urlpatterns = RegexURLMap()
urlpatterns.add_routes('', 
        #(r'^login/$', 'moviereviews.views.login', {
        #    'template_name' : 'moviereviews/login.mako'
        #}),

        url(r'^test_post$', \
            'moviereviews.views.test_schevo_database', {
            'template_name' : 'moviereviews/test.mako'
        }, name="test_post"),
)
urlpatterns.commit()

