# Suck in default django settings
from django.conf.global_settings import *

from pkg_resources import resource_filename

import os

DEBUG = True

# Keep this to False unless you expecting to debug I18N
USE_I18N = False

SECRET_KEY = 'secretkey'

# Custom error handlers mapping
CUSTOM_ERROR_HANDLERS = (
    #('handle401', 'sandbox.handlers.handle401'),
    #('handle403', 'sandbox.handlers.handle403'),
    ('handle404', 'sandbox.handlers.handle404'),
    ('handle500', 'sandbox.handlers.handle500')
    )

# List of processors used by RequestContext to populate the context.
# Each one should be a callable that takes the request object as its
# only parameter and returns a dictionary to add to the context.
TEMPLATE_CONTEXT_PROCESSORS = (
   'django.core.context_processors.request', # request
   'django.core.context_processors.media',
   'blogengine.contrib.addthis.context_processors.addthis_widget',  # addthis widget
   'django.contrib.auth.context_processors.auth'
   )
# For backward-compatibility with Django
TEMPLATE_DIRS = (
    resource_filename('sandbox', 'templates'),
)

ROOT_URLCONF = 'sandbox.config.urls'

MIDDLEWARE_CLASSES = ()

#SITE_ID = 1

#    )
INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.contenttypes'
    )
ENABLE_TIDYLIB = False
ENABLE_LOGGING = True
LOGGING_CALLBACK = 'rootLogger'

SCHEVO_DATABASE_NAME = 'schevo.db.blogs'

ADDTHIS_USERNAME = 'erob'

CACHE_BACKEND_LOCATION = '127.0.0.1:11221'
