from notmm.utils.urlmap import RegexURLMap

urlpatterns = RegexURLMap()

#urlpatterns.set_default_prefix('/foo/')

urlpatterns.add_routes('sandbox_sqlalchemy.views', 
        ### decorators tests
        ### with_schevo_database test
        (r'^test_with_session$', 'test_with_session', {
            'template_name' : 'sandbox/test.mako'
        }),
)

