#!/usr/bin/env python
from notmm.utils.configparse import setup_all
#import logging_conf

global_conf = {}

setup_all('sandbox_sqlalchemy', global_conf)
setup_all('template', global_conf)
app_conf = global_conf['sandbox_sqlalchemy']

